<!DOCTYPE html>
<html lang="zh-cn">
<head>
<meta charset="utf-8" />
<title></title>
<meta name="viewport" content="width=device-width,initial-scale=1.0,maximum-scale=1.0,user-scalable=no"/>
<meta name="HandheldFriendly" content="true"/>
<meta name="MobileOptimized" content="320"/>
<%include file="/app/common/html/include/static.html"/>
<style>
*{padding:0px;margin:0px}
.el-header, .el-footer {background-color: #B3C0D1;line-height: 60px;padding:0px;}
.kcws-side-scroll::-webkit-scrollbar{width:0px}
</style>
</head>
<body>
<div id="app">
    {{content}}<br>
    当前窗口宽高为:{{winwidth}}px,{{winheight}}px<br>
</div>
</body>
<script>
var VU=new Vue({
        el: '#app',
        data:{
            winheight:document.documentElement.clientHeight,winwidth:document.documentElement.clientWidth,
            content:'我是vue变量'
        },
        mounted:function(){
            self=this
            window.onresize = function(){
                self.winheight=document.documentElement.clientHeight
                self.winwidth=document.documentElement.clientWidth
            }
        },
        methods: {
            
        }
});
</script>
</html>
